/************************************************************************
 *
 * Basic Interval Arithmetic Subroutines Level 0
 * Modified for generalized intervals and Kaucher arithmetic (2005/06 A. Goldsztejn)
 * ---------------------------------------------
 *
 * Copyright (c) 1993, 1997 Olaf Knueppel
 *
 * Modifications for generalized intervals: Alexandre Goldsztejn (Alexandre@Goldsztejn.com)
 *
 ************************************************************************/

#ifndef __BIASK0__
#define __BIASK0__

#include "BiasKType.h"

#define __BIASKINFSUP__
#define __BIASKIEEE__

/************************************************************************
 *  Definition of Constants, Types, and Variables
 ************************************************************************/

extern REAL     BiasKEpsilon;            /* min {x >= 0 : 1 + x > 1}     */
extern REAL     BiasKEta;                /* min {x > 0}                  */
extern REAL     BiasKNaN;                /* NaN                          */
extern REAL     BiasKPosInf;             /* +oo                          */
extern REAL     BiasKNegInf;             /* -oo                          */

/************************************************************************
 *  Function Prototypes
 ************************************************************************/
#if defined (__cplusplus)
extern "C" {
#endif

VOID    BiasKRoundUp     (VOID);
VOID    BiasKRoundDown   (VOID);
VOID    BiasKRoundNear   (VOID);

VOID    BiasKInit        (VOID);

REAL    BiasKPredR       (const REAL         * const pa);
INT     BiasKPredI       (      BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA);

REAL    BiasKSuccR       (const REAL         * const pa);
// BiasK: new fonction
VOID    BiasKSuccR2I     (BIASKINTERVAL * const pR,
			 const REAL * const pa);
// ****************************************
VOID    BiasKSuccI       (BIASKINTERVAL       * const pR,
                         const BIASKINTERVAL * const pA);

VOID    BiasKAddRR       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const REAL         * const pb);
VOID    BiasKAddRI       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
VOID    BiasKAddII       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

VOID    BiasKSubRR       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const REAL         * const pb);
VOID    BiasKSubRI       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
VOID    BiasKSubIR       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const REAL         * const pb);
VOID    BiasKSubII       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

VOID    BiasKMulRR       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const REAL         * const pb);
VOID    BiasKMulRI       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
VOID    BiasKMulII       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

VOID    BiasKDivRR       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const REAL         * const pb);
VOID    BiasKDivRI       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
VOID    BiasKDivIR       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const REAL         * const pb);
VOID    BiasKDivII       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

VOID    BiasKMacRR       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const REAL         * const pb);
VOID    BiasKMacRI       (BIASKINTERVAL       * const pR,
			 const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
VOID    BiasKMacII       (BIASKINTERVAL       * const pR,
			 const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

VOID    BiasKNeg         (BIASKINTERVAL       * const pR,
                         const BIASKINTERVAL * const pA);

// BiasK: new fonctions in place of BiasAbs
// ****************************************
VOID BiasKDual (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA);
VOID BiasKPro  (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA);
VOID BiasKImp  (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA);
// ****************************************

REAL    BiasKMid         (const BIASKINTERVAL * const pA);
VOID    BiasKMidRad      (REAL               * const pm,
                         REAL               * const pr,
			 const BIASKINTERVAL * const pA);
REAL    BiasKDiam        (const BIASKINTERVAL * const pA);

REAL    BiasKAbs         (const BIASKINTERVAL * const pA);
// BiasK: new fonctions in place of BiasAbs
// ****************************************
REAL    BiasKMig         (const BIASKINTERVAL * const pA);

REAL    BiasKDistRI      (const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
REAL    BiasKDistII      (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

// BiasK: new fonctions
// ****************************************
VOID BiasKRR2I (BIASKINTERVAL * const pR,
		 const REAL * const pa, const REAL * const pb);
VOID BiasKMeetRR (BIASKINTERVAL * const pR,
		 const REAL * const pa, const REAL * const pb);
VOID BiasKMeetRI (BIASKINTERVAL * const pR,
		 const REAL * const pa, const BIASKINTERVAL * const pB);
VOID BiasKMeetII (BIASKINTERVAL * const pR,
		 const BIASKINTERVAL * const pA, const BIASKINTERVAL * const pB);

VOID BiasKJoinRR (BIASKINTERVAL * const pR,
		 const REAL * const pa, const REAL * const pb);
VOID BiasKJoinRI (BIASKINTERVAL * const pR,
		 const REAL * const pa, const BIASKINTERVAL * const pB);
VOID BiasKJoinII (BIASKINTERVAL * const pR,
		 const BIASKINTERVAL * const pA, const BIASKINTERVAL * const pB);
// ****************************************

INT     BiasKInR         (const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
INT     BiasKInI         (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

INT     BiasKInInteriorR (const REAL         * const pa,
                         const BIASKINTERVAL * const pB);
INT     BiasKInInteriorI (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);

INT     BiasKIsEqual     (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB);
// BiasK: new fonctions
// ****************************************
INT BiasKIsProper (const BIASKINTERVAL * const pA);
INT BiasKIsImproper (const BIASKINTERVAL * const pA);
// ****************************************


#define BiasKInf(pA)     ((pA)->inf)
#define BiasKSup(pA)     ((pA)->sup)

#define BiasKAddIR(pR, pA, pb)    BiasKAddRI (pR, pb, pA)
#define BiasKMulIR(pR, pA, pb)    BiasKMulRI (pR, pb, pA)
#define BiasKMacIR(pR, pA, pb)    BiasKMacRI (pR, pb, pA)
#define BiasKDistIR(pA, pb)       BiasKDistRI (pb, pA)
#define BiasKMeetIR(pR, pA, pb)   BiasKMeetRI (pR, pb, pA)
#define BiasKJoinIR(pR, pA, pb)   BiasKJoinRI (pR, pb, pA)
#define BiasKHullIR(pR, pA, pb)   BiasKHullRI (pR, pb, pA)

#if defined (__cplusplus)
}
#endif

#endif /* __BIASK0__ */
