/************************************************************************
 *
 * Basic Interval Arithmetic Subroutines Level 1
 * Modified for generalized intervals and Kaucher arithmetic (2005/06 A. Goldsztejn)
 * ---------------------------------------------
 *
 * Copyright (c) 1993, 1997 Olaf Knueppel
 *
 * $Id: BiasK1.h,v 1.1 1997/09/23 11:50:35 knueppel Exp $
 *
 ************************************************************************/

// Modified for generaliezd interval ( A. Goldsztejn 15.11.2004 )

#ifndef __BIASK1__
#define __BIASK1__

#include "BiasK0.h"

/************************************************************************
 *  Function Prototypes
 ************************************************************************/

#if defined (__cplusplus)
extern "C" {
#endif

VOID    BiasKPredVR      (REAL * const  pr,
			 const REAL * const  pa,
                         const INT dim);
INT     BiasKPredVI      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

VOID    BiasKSuccVR      (REAL * const  pr,
                         const REAL * const  pa,
                         const INT dim);
VOID 	BiasKSuccVR2VI 	(BIASKINTERVAL * const pR_start,
                 	 const REAL * const pa_start,
		 	 const int dim);
VOID    BiasKSuccVI      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

VOID    BiasKAddVRVR     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKAddVRVI     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKAddVIVI     (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);

VOID    BiasKSubVRVR     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKSubVRVI     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKSubVIVR     (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKSubVIVI     (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);

VOID    BiasKMulRVR      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKMulIVR      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const REAL * const pb,
                         const INT dim);
VOID    BiasKMulRVI      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKMulIVI      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pa,
                         const BIASKINTERVAL *pB,
                         const INT dim);

VOID    BiasKMacRVR      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKMacIVR      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const REAL * const pb,
                         const INT dim);
VOID    BiasKMacRVI      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKMacIVI      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL *pB,
                         const INT dim);

VOID    BiasKMacRVIs     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim,
                         const INT Bstep);
VOID    BiasKMacsRVIs    (BIASKINTERVAL * const pR,
			 const REAL * const pa,
                         const BIASKINTERVAL * const pB,
			 const INT dim,
                         const INT Rstep,
                         const INT Bstep);

VOID    BiasKDivVRR      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKDivVIR      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKDivVRI      (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKDivVII      (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL *pA,
			 const BIASKINTERVAL * const pB,
                         const INT dim);

VOID    BiasKMacVRVR     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const REAL * const pb,
                         const INT dim);
VOID    BiasKMacVRVI     (BIASKINTERVAL * const pR,
                         const REAL * const pa,
			 const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKMacVIVI     (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL *pB,
                         const INT dim);

VOID    BiasKInfV        (REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);
VOID    BiasKSupV        (REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

VOID    BiasKNegV        (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

// BiasKK: new fonctions
// ****************************************
VOID BiasKDualV (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA, const INT dim);
VOID BiasKProV (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA, const INT dim);
VOID BiasKImpV (BIASKINTERVAL * const pR, const BIASKINTERVAL * const pA, const INT dim);
// **************************************************************************

VOID    BiasKMidV        (REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);
VOID    BiasKMidRadV     (REAL * const pm,
                         REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);
VOID    BiasKDiamV       (REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

VOID    BiasKAbsV        (REAL * const pr,
                         const BIASKINTERVAL * const pA,
                         const INT dim);

// BiasKK; new functions
// **************************************************************************
VOID    BiasKVRVR2VI     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL *pb,
                         const INT dim);
VOID    BiasKMeetVRVR    (BIASKINTERVAL * const pR,
                         const REAL * const pa,
                         const REAL * const pb,
                         const INT dim);
VOID    BiasKMeetVRVI    (BIASKINTERVAL * const pR,
                         const REAL * const pa,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKMeetVIVI    (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKJoinVRVR    (BIASKINTERVAL * const pR,
                         const REAL * const pa,
                         const REAL * const pb,
                         const INT dim);
VOID    BiasKJoinVRVI    (BIASKINTERVAL * const pR,
                         const REAL * const pa,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
VOID    BiasKJoinVIVI    (BIASKINTERVAL * const pR,
                         const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
// **************************************************************************

INT     BiasKInVR        (const REAL * const pa,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
INT     BiasKInVI        (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB,
                         const INT dim);

INT     BiasKInInteriorVR (const REAL * const pa,
                         const BIASKINTERVAL * const pB,
                         const INT dim);
INT     BiasKInInteriorVI (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB,
                         const INT dim);

INT     BiasKIsEqualV    (const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL * const pB,
                         const INT dim);

VOID    BiasKSetToZeroV  (BIASKINTERVAL * const pR,
                         const INT dim);
VOID    BiasKSetToZeroVs (BIASKINTERVAL * const pR,
                         const INT dim,
                         const INT step);

// BiasK: new fonctions
// ****************************************
INT BiasKIsProperV (const BIASKINTERVAL * const pA, int dim);
INT BiasKIsImproperV (const BIASKINTERVAL * const pA, int dim);
// ****************************************

#define BiasKAddVIVR(pR, pA, pb, d)   BiasKAddVRVI (pR, pb, pA, d)
#define BiasKMacVIVR(pR, pA, pb, d)   BiasKMacVRVI (pR, pb, pA, d)
#define BiasKMeetVIVR(pR, pA, pb, d)  BiasKMeetVRVI (pR, pb, pA, d)
#define BiasKJoinVIVR(pR, pA, pb, d)  BiasKJoinVRVI (pR, pb, pA, d)
#define BiasKIntersectionVIVR(pR, pA, pb, d)  BiasKIntersectionVRVI (pR, pb, pA, d)

#if defined (__cplusplus)
}
#endif

#endif /* __BIASK1__ */
