/************************************************************************
 *
 * Basic Interval Arithmetic Subroutines Level 2
 * Modified for generalized intervals and Kaucher arithmetic (2005/06 A. Goldsztejn)
 * ---------------------------------------------
 *
 * Copyright (c) 1993, 1997 Olaf Knueppel
 *
 * $Id: BiasK2.h,v 1.1.1.1 2004/04/21 10:02:36 dalmas Exp $
 *
 ************************************************************************/

#ifndef __BIASK2__
#define __BIASK2__

#include "BiasK1.h"

/* #define __BIASK2USEVECTOROPERATIONS__ */

/************************************************************************
 *  Function Prototypes
 ************************************************************************/

#if defined (__cplusplus)
extern "C" {
#endif

VOID    BiasKMulRMR      (BIASKINTERVAL *pR,
                         const REAL * const pa,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulRMI      (BIASKINTERVAL *pR,
                         const REAL * const pa,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulIMR      (BIASKINTERVAL *pR,
                         const BIASKINTERVAL * const pA,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulIMI      (BIASKINTERVAL *pR,
                         const BIASKINTERVAL * const pA,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);

VOID    BiasKDivMRR      (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL * const pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKDivMRI      (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const BIASKINTERVAL * const pB,
			 const INT rows,
                         const INT cols);
VOID    BiasKDivMIR      (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const REAL * const pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKDivMII      (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL * const pB,
			 const INT rows,
                         const INT cols);

VOID    BiasKMulMRVR     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL * const pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulMRVI     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const BIASKINTERVAL * const pB,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulMIVR     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKMulMIVI     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL * const pB,
			 const INT rows,
                         const INT cols);

#if defined (__BIASK2USEVECTOROPERATIONS__)
#define BiasKAddMRMR(pR, pa, pb, r, c)      BiasKAddVRVR (pR, pa, pb, (r)*(c))
#define BiasKAddMRMI(pR, pa, pB, r, c)      BiasKAddVRVI (pR, pa, pB, (r)*(c))
#define BiasKAddMIMR(pR, pA, pb, r, c)      BiasKAddVIVR (pR, pA, pb, (r)*(c))
#define BiasKAddMIMI(pR, pA, pB, r, c)      BiasKAddVIVI (pR, pA, pB, (r)*(c))
#define BiasKSubMRMR(pR, pa, pb, r, c)      BiasKSubVRVR (pR, pa, pb, (r)*(c))
#define BiasKSubMRMI(pR, pa, pB, r, c)      BiasKSubVRVI (pR, pa, pB, (r)*(c))
#define BiasKSubMIMR(pR, pA, pb, r, c)      BiasKSubVIVR (pR, pA, pb, (r)*(c))
#define BiasKSubMIMI(pR, pA, pB, r, c)      BiasKSubVIVI (pR, pA, pB, (r)*(c))
#else

VOID    BiasKAddMRMR     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKAddMRMI     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);
#define BiasKAddMIMR(pR, pA, pb, r, c)      BiasKAddMRMI (pR, pb, pA, r, c)
VOID    BiasKAddMIMI     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);

VOID    BiasKSubMRMR     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKSubMRMI     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);
VOID    BiasKSubMIMR     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const REAL *pb,
			 const INT rows,
                         const INT cols);
VOID    BiasKSubMIMI     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL *pB,
			 const INT rows,
                         const INT cols);
#endif

VOID    BiasKMulMRMR     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const REAL *pb,
			 const INT arows,
                         const INT acols,
                         const INT bcols);
VOID    BiasKMulMRMI     (BIASKINTERVAL *pR,
                         const REAL *pa,
                         const BIASKINTERVAL * const pB,
			 const INT arows,
                         const INT acols,
                         const INT Bcols);
VOID    BiasKMulMIMR     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL * const pA,
                         const REAL *pb,
			 const INT Arows,
                         const INT Acols,
                         const INT bcols);
VOID    BiasKMulMIMI     (BIASKINTERVAL *pR,
                         const BIASKINTERVAL *pA,
                         const BIASKINTERVAL * const pB,
			 const INT Arows,
                         const INT Acols,
                         const INT Bcols);

#define BiasKPredMR(pr, pa, r, c)           BiasKPredVR (pr, pa, (r)*(c))
#define BiasKPredMI(pR, pA, r, c)           BiasKPredVI (pR, pA, (r)*(c))
#define BiasKSuccMR(pr, pa, r, c)           BiasKSuccVR (pr, pa, (r)*(c))
#define BiasKSuccMI(pR, pA, r, c)           BiasKSuccVI (pR, pA, (r)*(c))

#define BiasKInfM(pr, pA, r, c)             BiasKInfV (pr, pA, (r)*(c))
#define BiasKSupM(pr, pA, r, c)             BiasKSupV (pr, pA, (r)*(c))

#define BiasKNegM(pR, pA, r, c)             BiasKNegV (pR, pA, (r)*(c))

#define BiasKMidM(pr, pA, r, c)             BiasKMidV (pr, pA, (r)*(c))
#define BiasKMidRadM(pm, pr, pA, r, c)      BiasKMidRadV (pm, pr, pA, (r)*(c))

#define BiasKDiamM(pr, pA, r, c)            BiasKDiamV (pr, pA, (r)*(c))

#define BiasKAbsM(pr, pA, r, c)             BiasKAbsV (pr, pA, (r)*(c))

#define BiasKDualM(pR, pa, r, c)            BiasKDualV (pR, pa, (r)*(c))
#define BiasKProM(pR, pa, r, c)             BiasKProV (pR, pa, (r)*(c))
#define BiasKImpM(pR, pa, r, c)             BiasKImpV (pR, pa, (r)*(c))

#define BiasKMeetMRMR(pR, pa, pb, r, c)     BiasKMeetVRVR (pR, pa, pb, (r)*(c))
#define BiasKMeetMRMI(pR, pa, pB, r, c)     BiasKMeetVRVI (pR, pa, pB, (r)*(c))
#define BiasKMeetMIMR(pR, pA, pb, r, c)     BiasKMeetVIVR (pR, pA, pb, (r)*(c))
#define BiasKMeetMIMI(pR, pA, pB, r, c)     BiasKMeetVIVI (pR, pA, pB, (r)*(c))

#define BiasKJoinMRMR(pR, pa, pb, r, c)     BiasKJoinVRVR (pR, pa, pb, (r)*(c))
#define BiasKJoinMRMI(pR, pa, pB, r, c)     BiasKJoinVRVI (pR, pa, pB, (r)*(c))
#define BiasKJoinMIMR(pR, pA, pb, r, c)     BiasKJoinVIVR (pR, pA, pb, (r)*(c))
#define BiasKJoinMIMI(pR, pA, pB, r, c)     BiasKJoinVIVI (pR, pA, pB, (r)*(c))

#define BiasKInMR(pa, pB, r, c)             BiasKInVR (pa, pB, (r)*(c))
#define BiasKInMI(pA, pB, r, c)             BiasKInVI (pA, pB, (r)*(c))

#define BiasKInInteriorMR(pa, pB, r, c)     BiasKInInteriorVR (pa, pB, (r)*(c))
#define BiasKInInteriorMI(pA, pB, r, c)     BiasKInInteriorVI (pA, pB, (r)*(c))

#define BiasKIsEqualM(pA, pB, r, c)         BiasKIsEqualV (pA, pB, (r)*(c))

#define BiasKSetToZeroM(pR, r, c)           BiasKSetToZeroV (pR, (r)*(c))

#if defined (__cplusplus)
}
#endif

#endif /* __BIASK2__ */
