/************************************************************************
 *
 * Basic Interval Arithmetic Subroutines Level 2
 * Modified for generalized intervals and Kaucher arithmetic (2005/06 A. Goldsztejn)
 * ---------------------------------------------
 *
 * Copyright (c) 1993, 1997 Olaf Knueppel
 *
 * $Id: BiasK2.c,v 1.1.1.1 2004/04/21 10:02:35 dalmas Exp $
 *
 ************************************************************************/

#include "BiasK2.h"
#include "BiasRnd.h"
#include "BiasKInt.h"

#if !defined (__BIASKINFSUP__) || !defined (__BIASKIEEE__)
#error __BIASKINFSUP__ and __BIASKIEEE__ not set
#endif

/************************************************************************
 *  Implementation of global functions
 ************************************************************************/

VOID BiasKMulRMR (BIASKINTERVAL *pR,
                 const REAL * const pa,
                 const REAL *pb,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = a * mb
   */
{
  INT i;

  for (i = 0; i < rows; i++, pb += cols, pR += cols)
    BiasKMulRVR (pR, pa, pb, cols);
}

VOID BiasKMulRMI (BIASKINTERVAL *pR,
                 const REAL * const pa,
                 const BIASKINTERVAL *pB,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = a * mB
   *  Assertion: mR != mB
   */
{
  INT i;

  for (i = 0; i < rows; i++, pB += cols, pR += cols)
    BiasKMulRVI (pR, pa, pB, cols);
}

VOID BiasKMulIMR (BIASKINTERVAL *pR,
                 const BIASKINTERVAL * const pA,
                 const REAL *pb,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = A * mb
   */
{
  INT i;

  for (i = 0; i < rows; i++, pb += cols, pR += cols)
    BiasKMulIVR (pR, pA, pb, cols);
}

VOID BiasKMulIMI (BIASKINTERVAL *pR,
                 const BIASKINTERVAL * const pA,
                 const BIASKINTERVAL *pB,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = A * mB
   *  Assertion: mR != mB
   */
{
  INT i;

  for (i = 0; i < rows; i++, pB += cols, pR += cols)
    BiasKMulIVI (pR, pA, pB, cols);
}

VOID BiasKDivMRR (BIASKINTERVAL *pR,
                 const REAL *pa,
                 const REAL * const pb,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = ma / b
   */
{
  INT i;

  for (i = 0; i < rows; i++, pa += cols, pR += cols)
    BiasKDivVRR (pR, pa, pb, cols);
}

VOID BiasKDivMRI (BIASKINTERVAL *pR,
                 const REAL *pa,
                 const BIASKINTERVAL * const pB,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = ma / B
   *  Assertion: mR != B
   */
{
  INT i;

  for (i = 0; i < rows; i++, pa += cols, pR += cols)
    BiasKDivVRI (pR, pa, pB, cols);
}

VOID BiasKDivMIR (BIASKINTERVAL *pR,
                 const BIASKINTERVAL *pA,
                 const REAL * const pb,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = mA / b
   *  Assertion: mR != mA
   */
{
  INT i;

  for (i = 0; i < rows; i++, pA += cols, pR += cols)
    BiasKDivVIR (pR, pA, pb, cols);
}

VOID BiasKDivMII (BIASKINTERVAL *pR,
                 const BIASKINTERVAL *pA,
                 const BIASKINTERVAL * const pB,
		 const INT rows,
                 const INT cols)
  /**********************************************************************
   *  mR = MA / B
   *  Assertion: mR != mA and mR != B
   */
{
  INT i;

  for (i = 0; i < rows; i++, pA += cols, pR += cols)
    BiasKDivVII (pR, pA, pB, cols);
}

VOID BiasKMulMRVR (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const REAL * const pb,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  vR = ma * vb
   */
{
  INT i;

  for (i = 0; i < rows; i++, pa += cols)
    BiasKMacVRVR (pR++, pa, pb, cols);
}

VOID BiasKMulMRVI (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const BIASKINTERVAL * const pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  vR = ma * vB
   */
{
  INT i;

  for (i = 0; i < rows; i++, pa += cols)
    BiasKMacVRVI (pR++, pa, pB, cols);
}

VOID BiasKMulMIVR (BIASKINTERVAL * const pR,
                  const BIASKINTERVAL *pA,
                  const REAL *pb,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  vR = mA * vb
   */
{
  INT i;

  BiasKSetToZeroV (pR, rows);
  for (i = 0; i < cols; i++, pb++)
    BiasKMacRVIs (pR, pb, pA++, rows, cols);
}

VOID BiasKMulMIVI (BIASKINTERVAL *pR,
                  const BIASKINTERVAL *pA,
                  const BIASKINTERVAL * const pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  vR = mA * vB
   */
{
  INT i;

  for (i = 0; i < rows; i++, pA += cols)
    BiasKMacVIVI (pR++, pA, pB, cols);
}

#if !defined (__BIASK2USEVECTOROPERATIONS__)

VOID BiasKAddMRMR (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const REAL *pb,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = ma + mb
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKAddVRVR (pR, pa, pb, cols);
    pR += cols; pa += cols; pb += cols;
  }
}

VOID BiasKAddMRMI (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const BIASKINTERVAL *pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = ma + mB
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKAddVRVI (pR, pa, pB, cols);
    pR += cols; pa += cols; pB += cols;
  }
}

VOID BiasKAddMIMI (BIASKINTERVAL *pR,
                  const BIASKINTERVAL *pA,
                  const BIASKINTERVAL *pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = mA + mB
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKAddVIVI (pR, pA, pB, cols);
    pR += cols; pA += cols; pB += cols;
  }
}

VOID BiasKSubMRMR (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const REAL *pb,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = ma - ma
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKSubVRVR (pR, pa, pb, cols);
    pR += cols; pa += cols; pb += cols;
  }
}

VOID BiasKSubMRMI (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const BIASKINTERVAL *pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = ma - mB
   *  Assertion: mR != mB
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKSubVRVI (pR, pa, pB, cols);
    pR += cols; pa += cols; pB += cols;
  }
}

VOID BiasKSubMIMR (BIASKINTERVAL *pR,
                  const BIASKINTERVAL *pA,
                  const REAL *pb,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = mA - mb
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKSubVIVR (pR, pA, pb, cols);
    pR += cols; pA += cols; pb += cols;
  }
}

VOID BiasKSubMIMI (BIASKINTERVAL *pR,
                  const BIASKINTERVAL *pA,
                  const BIASKINTERVAL *pB,
		  const INT rows,
                  const INT cols)
  /**********************************************************************
   *  mR = mA - mB
   *  Assertion: mR != mB
   */
{
  INT i;

  for (i = 0; i < rows; i++) {
    BiasKSubVIVI (pR, pA, pB, cols);
    pR += cols; pA += cols; pB += cols;
  }
}
#endif /* __BIASK2USEVECTOROPERATIONS__ */

#if !defined (_AIX)

VOID BiasKMulMRMR (BIASKINTERVAL * const pR_start,
                  const REAL *pa,
                  const REAL *pb,
		  const INT arows,
                  const INT acols,
                  const INT bcols)
  /**********************************************************************
   *  mR = ma * mb
   */
{
  INT i, j, k;
  REAL s;
  BIASKINTERVAL *pR;

  _BiasKRoundDown ();
  pR = pR_start;
  for (i = 0; i < arows; i++)
    for (j = 0; j < bcols; j++) {
      s = 0.0;
      for (k = 0; k < acols; k++)
	s += pa[i * acols + k] * pb[k * bcols + j];
      (pR++)->inf = s;
    }

  _BiasKRoundUp ();
  pR = pR_start;
  for (i = 0; i < arows; i++)
    for (j = 0; j < bcols; j++) {
      s = 0.0;
      for (k = 0; k < acols; k++)
	s += pa[i * acols + k] * pb[k * bcols + j];
      (pR++)->sup = s;
    }
}

#else

VOID BiasKMulMRMR (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const REAL *pb_start,
		  const INT arows,
                  const INT acols,
                  const INT bcols)
  /**********************************************************************
   *  mR = ma * mb
   */
{
  INT i, j;
  const REAL *pb;

  for (i = 0; i < arows; i++) {
    BiasKSetToZeroV (pR, bcols);
    pb = pb_start;
    for (j = 0; j < acols; j++) {
      BiasKMacRVR (pR, pa++, pb, bcols);
      pb += bcols;
    }
    pR += bcols;
  }
}

#endif /* !_AIX */

VOID BiasKMulMRMI (BIASKINTERVAL *pR,
                  const REAL *pa,
                  const BIASKINTERVAL * const pB_start,
		  const INT arows,
                  const INT acols,
                  const INT Bcols)
  /**********************************************************************
   *  mR = ma * mB
   */
{
  INT i, j;
  const BIASKINTERVAL *pB;

  for (i = 0; i < arows; i++) {
    BiasKSetToZeroV (pR, Bcols);
    pB = pB_start;
    for (j = 0; j < acols; j++) {
      BiasKMacRVI (pR, pa++, pB, Bcols);
      pB += Bcols;
    }
    pR += Bcols;
  }
}

VOID BiasKMulMIMR (BIASKINTERVAL *pR,
                  const BIASKINTERVAL * const pA_start,
                  const REAL *pb_start,
		  const INT Arows,
                  const INT Acols,
                  const INT bcols)
  /**********************************************************************
   *  mR = mA * mb
   */
{
  INT i, j;
  const REAL *pb;
  const BIASKINTERVAL *pA;

  for (i = 0; i < bcols; i++) {
    BiasKSetToZeroVs (pR, Arows, bcols);
    pb = pb_start;
    pA = pA_start;
    for (j = 0; j < Acols; j++) {
      BiasKMacsRVIs (pR, pb, pA++, Arows, bcols, Acols);
      pb += bcols;
    }
    pR++; pb_start++;
  }
}

VOID BiasKMulMIMI (BIASKINTERVAL *pR,
                  const BIASKINTERVAL *pA,
                  const BIASKINTERVAL * const pB_start,
		  const INT Arows,
                  const INT Acols,
                  const INT Bcols)
  /**********************************************************************
   *  mR = mA * mB
   */
{
  INT i, j;
  const BIASKINTERVAL *pB;

  for (i = 0; i < Arows; i++) {
    BiasKSetToZeroV (pR, Bcols);
    pB = pB_start;
    for (j = 0; j < Acols; j++) {
      BiasKMacIVI (pR, pA++, pB, Bcols);
      pB += Bcols;
    }
    pR += Bcols;
  }
}
