/************************************************************************
 *
 * Basic Interval Arithmetic Subroutines Level U
 * Utilitiy functions
 * ---------------------------------------------
 *
 *
 * Alexandre Goldsztejn (Alexandre@Goldsztejn.com)
 *
************************************************************************/

#include "BiasKU.h"
#include <stdio.h>

void printI(const BIASKINTERVAL *V)
{
  printf("[%f , %f]\n",V->inf,V->sup);
}

void printVI(const BIASKINTERVAL *V)
{
  printf("[%f , %f] ",V->inf,V->sup);
  V++;
  printf("[%f , %f]\n",V->inf,V->sup);
}

void printMI(const BIASKINTERVAL *M)
{
  printVI(M);
  M+=2;
  printVI(M);
}

void BiasKInverseDiagonalDecomposition(
        const BIASKINTERVAL *A, BIASKINTERVAL *D, BIASKINTERVAL *E, int dim)
{
  int i,dim2=dim*dim;
  double One=1.0;

  for(i=0 ; i<dim2 ; i++,A++,D++,E++)
  {
    if(i%(dim+1)==0) // on the diagonal
    {
        BiasKDivRI(D,&One,A);
        E->inf=0.0;
        E->sup=0.0;
    }else{
        D->inf=0.0;
        D->sup=0.0;
        E->inf=A->inf;
        E->sup=A->sup;
    }
  }
}

void BiasKMinusId(BIASKINTERVAL *AmI, const BIASKINTERVAL *A, int dim)
{
  int i,dim2=dim*dim;
  double One=1.0;

  for(i=0 ; i<dim2 ; i++,A++,AmI++)
  {
    if(i%(dim+1)==0) // on the diagonal
    {
        BiasKSubIR(AmI,A,&One);
    }else{
        AmI->inf=A->inf;
        AmI->sup=A->sup;
    }
  }
}

void BiasKFormalSolveLinearEquation(
        const BIASKINTERVAL *A, const BIASKINTERVAL *X0, const BIASKINTERVAL *B,
        int dim, int N_iter_max, double epsilon, BIASKINTERVAL *X)
// computes X which appromatively satisfies A X = B
// (problem dimension => dim)
// iterative method:
//      starting point   => X0
//      stoping criteria => k<N_ITER_MAX && wid(Xk+1)<epsilon wid(Xk)
// property: if the algorithm converges then A X_infty = B
{
  BIASKINTERVAL D[dim*dim],E[dim*dim];
  BIASKINTERVAL Xk[dim],T1[dim],T2[dim];
  int i,size_memcpy=dim*sizeof(BIASKINTERVAL);

  BiasKInverseDiagonalDecomposition(A,D,E,dim);
  BiasKDualV(D,D,dim*dim);
  if(X!=X0) memcpy(X,X0,size_memcpy);
  i=1;
  do{
    memcpy(Xk,X,size_memcpy);
    BiasKMulMIVI(T1,E,Xk,dim,dim);
    BiasKDualV(T2,T1,dim);
    BiasKSubVIVI(T1,B,T2,dim);
    BiasKMulMIVI(X,D,T1,dim,dim);
  }while(i++ <= N_iter_max);
}

int BiasKLinearKrawczyk(
        const BIASKINTERVAL *A, const BIASKINTERVAL *X0, const BIASKINTERVAL *B,
        int dim, int N_iter_max, double epsilon, BIASKINTERVAL *X)
{
  BIASKINTERVAL AmI[dim*dim];
  BIASKINTERVAL Xk[dim],T1[dim];
  int i,size_memcpy=dim*sizeof(BIASKINTERVAL);

  BiasKMinusId(AmI,A,dim);
  if(X!=X0) memcpy(X,X0,size_memcpy);
  i=1;
  do{
    memcpy(Xk,X,size_memcpy);
    BiasKMulMIVI(T1,AmI,Xk,dim,dim);
    BiasKSubVIVI(X,B,T1,dim);
    BiasKMeetVIVI(X,Xk,X,dim);
  }while(i++ <= N_iter_max & BiasKIsProperV(X,dim));

  if(BiasKIsProperV(X,dim))
    return 0;
  else
    return i;
}

void BiasKLU(BIASKINTERVAL *LU, const BIASKINTERVAL *A, int dim)
{
  int i,dim2=dim*dim;
  BIASKINTERVAL *LUT=LU;

  for(i=0 ; i<dim2 ; i++,A++,LU++)
  {
    if(i%(dim+1)==0) // on the diagonal
    {
      LU->inf=0.0;
      LU->sup=0.0;
    }else{
      LU->inf=A->inf;
      LU->sup=A->sup;
    }
  }
}

void BiasKLinearGaussSeidel_1(
        BIASKINTERVAL *X, BIASKINTERVAL *A, BIASKINTERVAL *T)
{
//    BIASKINTERVAL TT;
     if(! (A->inf<=0 && A->sup>=0)){
        BiasKDivII(T,T,A);
        BiasKMeetII(X,T,X);}
//        BiasKDivII(&TT,T,A);
//        BiasKMeetII(X,&TT,X);}
}

int BiasKLinearGaussSeidel_n(
        BIASKINTERVAL *A, BIASKINTERVAL *X, BIASKINTERVAL *B,
        int dim)
{
    int isProper,i,j;
    BIASKINTERVAL T,Aii,*Xi,*Xj;

    isProper=1;
    for(Xi=X,i=0;(i<dim)&&isProper;B++,Xi++,i++)
    {
        T=(*B);
        for(Xj=X,j=0;j<dim;A++,Xj++,j++)
        {
               if(i==j)
                  Aii= (*A);
                else
                  BiasKMacII(&T,Xj,A);
        }
        BiasKLinearGaussSeidel_1(Xi,&Aii,&T);
        if(!BiasKIsProper(Xi)) isProper=0;
    }    
    return isProper;
}


