%{
#include "calculator.h"

#define YY_ALWAYS_INTERACTIVE 1

extern char *yytext;
extern int *yyleng;
extern int yylex(void);
extern int yyerror(char*);
bool putmark;

int findVar(char*);
void setVarValue(char*, BIASKINTERVAL*);
BIASKINTERVAL* getVarValue(char*);
BIASKINTERVAL* myInterval(double, double);
BIASKINTERVAL* myExpresion(int op, BIASKINTERVAL*, BIASKINTERVAL*);
BIASKINTERVAL* myFunction(char*, BIASKINTERVAL*, double);
BIASKINTERVAL* meet(BIASKINTERVAL*, BIASKINTERVAL*);
BIASKINTERVAL* join(BIASKINTERVAL*, BIASKINTERVAL*);
void printI(BIASKINTERVAL*);
void msghelp();

vector<Vars> variables;
char* functions[] = {"sqr", "sqrt", "root", "cos", "sin", "exp", "log",
                     "tan", "arcsin", "arccos", "arctan", "abs", 
                     "Dual", "Pro", "Mid"};
int Nfunc = 15;

ostream& operator<<(ostream& os,const BIASKINTERVAL &i){
  os<<"["<<i.inf<<", "<<i.sup<<"]";
  return os;
}

%}
%union{
  double real;
  char* id;
  BIASKINTERVAL* itv;
}
%token<id> VAR FUNC
%token<real> NUM
%token ASIG APAR CPAR ACOR CCOR PCOM COMA MEET JOIN HELP QUIT
%left MAS MEN 
%left MUL DIV
%right ELEVA
%type<itv> intervalo expresion valor funcion linea
%type<real> areal
%start programa
%%  
programa: programa linea PCOM   { printI($2); }
        | programa HELP         { msghelp();  }
        | programa QUIT         { cout<<"--\nFinish!\n\n"; exit(1);}
        | linea PCOM            { printI($1); }
        | error PCOM            { yyerrok; cout<<"\n-> "<<flush;  }
        | QUIT                  { cout<<"--\nFinish!\n\n"; exit(1);}
        | HELP                  { msghelp();  }
;

linea   : asignacion            { $$ = NULL;  }
	| expresion		{ $$ = $1;    }
;

asignacion: VAR ASIG expresion     { setVarValue($1, $3);         }
;

funcion: FUNC APAR expresion CPAR  { $$ = myFunction($1, $3, 0);  }
         | FUNC APAR expresion COMA NUM CPAR { $$ = myFunction($1, $3, $5);}
         | MEET APAR expresion COMA expresion CPAR { $$ = meet($3, $5);}
         | JOIN APAR expresion COMA expresion CPAR { $$ = join($3, $5);}
;

expresion: valor                     { $$ = $1;                     }
         | funcion                   { $$ = $1;                     }
	 | expresion MAS expresion   { $$ = myExpresion(1, $1, $3); } 
	 | expresion MEN expresion   { $$ = myExpresion(2, $1, $3); }
         | expresion MUL expresion   { $$ = myExpresion(3, $1, $3); }
         | expresion DIV expresion   { $$ = myExpresion(4, $1, $3); }
	 | expresion ELEVA expresion { $$ = myExpresion(5, $1, $3); }
	 | APAR expresion CPAR       { $$ = $2;                     }
;

intervalo: ACOR areal COMA areal CCOR     { $$ = myInterval($2, $4); }
	 | MEN ACOR areal COMA areal CCOR { $$ = myInterval(-$5, -$3); } 
;

areal: NUM		   { $$ = $1; }
     | MEN NUM		   { $$ = - $2; }
;

valor: areal		   { $$ = myInterval($1, $1); }
     | intervalo           { $$ = $1;                 }
     | VAR                 { $$ = getVarValue($1);    }
;
%%

void printI(BIASKINTERVAL* i){
  cout.precision(12);
  if (i != NULL)
    cout<<"= "<<(*i)<<endl;
  if (putmark)
    cout<<endl<<"-> "<<flush;
  delete i;
}

BIASKINTERVAL* myInterval(double inf, double sup){
  BIASKINTERVAL* itv = new BIASKINTERVAL;
  itv->inf = inf;
  itv->sup = sup;
  return itv;
}


BIASKINTERVAL* myExpresion(int op, BIASKINTERVAL* i1, BIASKINTERVAL* i2){
  BIASKINTERVAL* res = new BIASKINTERVAL;
  res->inf = res->sup = 0;
  switch (op){
    case 1: BiasKAddII(res, i1, i2); break;
    case 2: BiasKSubII(res, i1, i2); break;
    case 3: BiasKMulII(res, i1, i2); break;
    case 4: BiasKDivII(res, i1, i2); break;
    case 5: if (i2->inf == i2->sup) 
               BiasKPowerN(res, i1, (int) i2->inf);
            else 
               cout<<"Error!, interval power does not allowed."<<endl;
            break;
    default: cout<<"Error!, operations does not implemented."<<endl; exit(1);
  }   
  delete i1; delete i2;
  return res;
}

int findVar(char* name){
  int nvars = variables.size();
  for (int i=0; i<nvars; i++)
    if (!strcmp(variables[i].name, name))
       return i;
  return -1;
}

void setVarValue(char* name, BIASKINTERVAL* i1){
  int pos = findVar(name);
  if (pos == -1){
     Vars v1;
     v1.name = new char[strlen(name) + 1];
     strcpy(v1.name, name);
     v1.val = i1;
     variables.push_back(v1);
  } else {
     delete variables[pos].val;
     variables[pos].val = i1;
  }
  cout<<name<<" = "<<(*i1)<<endl;
}


BIASKINTERVAL* getVarValue(char* name){
  BIASKINTERVAL* res = new BIASKINTERVAL;
  res->inf = res->sup = 0;
  int pos = findVar(name);
  if (pos == -1)
     cout<<"Warning!, using variable '"<<name<<"' without declaring."<<endl;
  else 
     (*res) = (*variables[pos].val);
  return res;
}


BIASKINTERVAL* myFunction(char* name, BIASKINTERVAL* i1, double val = 1){
  BIASKINTERVAL* res = new BIASKINTERVAL;
  res->inf = 2; res->sup = 3;
  int op = -1;
  for (int i=0; i<Nfunc; i++)
    if (!strcmp(functions[i], name)){
       op = i; break;
    }
  switch (op){
    case 0: BiasKSqr(res, i1); break;
    case 1: BiasKSqrt(res, i1); break;
    case 2: if (int(val) == val) { // (*res) = Root(*i1, val); 
               cout<<"Warning!, not implemented."<<endl;
               res->inf = 0; res->sup = 0; }
            else 
               cout<<"Warning!, using double variable in a integer place."<<endl;
            break;
    case 3: BiasKCos(res, i1); break;
    case 4: BiasKSin(res, i1); break;
    case 5: BiasKExp(res, i1); break;
    case 6: BiasKLog(res, i1); break;
    case 7: BiasKTan(res, i1); break;
    case 8: BiasKArcSin(res, i1); break;
    case 9: BiasKArcCos(res, i1); break;
    case 10: BiasKArcTan(res, i1); break;
    case 11: BiasKIAbs(res, i1); break;

    case 12: BiasKDual(res, i1); break;
    case 13: BiasKPro(res, i1); break;
    case 14: res->inf = res->sup = BiasKMid(i1); break;
    default: cout<<"Error!, operations does not implemented."<<endl; exit(1);
  }   
  delete i1;
  return res;
}

BIASKINTERVAL* meet(BIASKINTERVAL* i1, BIASKINTERVAL* i2){
  BIASKINTERVAL* res = new BIASKINTERVAL;
  BiasKMeetII(res, i1, i2);
  delete i1;   delete i2;
  return res;
}

BIASKINTERVAL* join(BIASKINTERVAL* i1, BIASKINTERVAL* i2){
  BIASKINTERVAL* res = new BIASKINTERVAL;
  BiasKJoinII(res, i1, i2);
  delete i1;   delete i2;
  return res;
}

void msghelp(){
  cout<<"\n\n--------------------------------------------------------------\n";
  cout<<"Operators:  +, -, *, /, ^                                    \n";
  cout<<"Functions:  sqr(), sqrt(), root(), exp(), log(), abs(), sin(),\n";
  cout<<"            cos(), tan(), arcsin(), arccos(), arctan().       \n";
  cout<<"Specials :  Mid(), Dual(), Pro(), Meet(_,_), Join(_,_).     \n\n";
  cout<<"(Memory implemented by assignment, e.g.: x = [1,5];)          \n";
  cout<<"--------------------------------------------------------------\n";
  cout<<endl<<"-> "<<flush;
}
